/*
 * Copyright (C) 2003 Ichiro Satoh (ichiro@nii.ac.jp)
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting
 * documentation.
 *
 * FREE USE OF THIS SOFTWARE IS ALLOWED IN ITS "AS IS" CONDITION.
 * BUT WITHOUT WARRANTY. THE AUTHORS DISCLAIM ANY LIABILITY OF ANY 
 * KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS 
 * SOFTWARE.
 */
 
package tinyrmi;

import java.io.Serializable;

public class ObjectId implements Serializable {
  private static long lastTime = System.currentTimeMillis();
  private static short lastCount = Short.MIN_VALUE;
  private static long ONE_SECOND = 1000; // in milliseconds
  
  private int unique;
  private long time;
  private short count;

  public static void main(String[] args) {
    try {
      System.out.println("> "+(new ObjectId()).toString());
      System.out.println("> "+(new ObjectId()).toString());
      System.out.println("> "+(new ObjectId()).toString());
      System.out.println("> "+(new ObjectId()).toString());
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  public ObjectId(int unique, long time, short count) {
    this.unique = unique;
    this.time = time;
    this.count = count;
  }

  public ObjectId(Object obj) {
    this();
    this.unique = getHashCode(obj);
  }

  public ObjectId() {
    synchronized (this) {
      if (lastCount == Short.MAX_VALUE) {
	lastCount = Short.MIN_VALUE;
      }
      time = System.currentTimeMillis();
      unique = getUniqueHashCode();
      count = lastCount++;
    }
  }
  
  private static int getHashCode(Object obj) {
    return obj.hashCode();
  }

  private static int getUniqueHashCode() {
    return getHashCode(new Object());
  }
  
  public int hashCode() {
    return (int)time + (int)count;
  }

  public boolean equals(Object obj) {
    if ((obj != null) && (obj instanceof ObjectId)) {
      ObjectId objid = (ObjectId)obj;
      return (unique == objid.unique &&
	      count == objid.count &&
	      time == objid.time);
    } 
    else {
      return false;
    }
  }
  
  public String toString() {
    return Integer.toString(unique,16) + ":" +
      Long.toString(time,16) + ":" +
	Integer.toString(count,16);
  }
}
