/*
 * Copyright (C) 2003 Ichiro Satoh (ichiro@nii.ac.jp)
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting
 * documentation.
 *
 * FREE USE OF THIS SOFTWARE IS ALLOWED IN ITS "AS IS" CONDITION.
 * BUT WITHOUT WARRANTY. THE AUTHORS DISCLAIM ANY LIABILITY OF ANY 
 * KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS 
 * SOFTWARE.
 */
 
package tinyrmi;

import java.io.Serializable;

public class Reply implements Serializable {
  private Object theReply = null;
  private Object theType = null;
  private Exception theException = null;

  public Reply() {}

  public Reply(Object reply) {
    this();
    theReply = reply;
  }

  public void setReply(Object reply) {
    theType = new Byte(Type.TYPE_OBJECT);
    theReply = reply;
  }

  public void setReply() {
    theType = new Byte(Type.TYPE_VOID);
  }

  public void setReply(boolean value) {
    theType = new Byte(Type.TYPE_BOOLEAN);
    theReply = new Boolean(value);
  }

  public void setReply(byte value) {
    theType = new Byte(Type.TYPE_BYTE);
    theReply = new Byte(value);
  }

  public void setReply(short value) {
    theType = new Byte(Type.TYPE_SHORT);
    theReply = new Short(value);
  }

  public void setReply(char value) {
    theType = new Byte(Type.TYPE_CHAR);
    theReply = new Character(value);
  }

  public void setReply(int value) {
    theType = new Byte(Type.TYPE_INT);
    theReply = new Integer(value);
  }

  public void setReply(long value) {
    theType = new Byte(Type.TYPE_LONG);
    theReply = new Long(value);
  }

  public void setReply(float value) {
    theType = new Byte(Type.TYPE_FLOAT);
    theReply = new Float(value);
  }

  public void setReply(double value) {
    theType = new Byte(Type.TYPE_DOUBLE);
    theReply = new Double(value);
  }

  public Object getReply() {
    return theReply;
  }

  public Object getObject() {
    return theReply;
  }

  public boolean booleanValue() throws IllegalAccessException {
    if (((Byte)theType).byteValue() != Type.TYPE_BOOLEAN) {
      throw new IllegalAccessException("IllegalType: boolean");
    }
    return ((Boolean)theReply).booleanValue();
  }

  public byte byteValue() throws IllegalAccessException {
    if (((Byte)theType).byteValue() != Type.TYPE_BYTE) {
      throw new IllegalAccessException("IllegalType: byte");
    }
    return ((Byte)theReply).byteValue();
  }

  public short shortValue() throws IllegalAccessException {
    if (((Byte)theType).byteValue() != Type.TYPE_SHORT) {
      throw new IllegalAccessException("IllegalType: short");
    }
    return ((Short)theReply).shortValue();
  }

  public char charValue() throws IllegalAccessException {
    if (((Byte)theType).byteValue() != Type.TYPE_CHAR) {
      throw new IllegalAccessException("IllegalType: char");
    }
    return ((Character)theReply).charValue();
  }

  public int intValue() throws IllegalAccessException {
    if (((Byte)theType).byteValue() != Type.TYPE_INT) {
      throw new IllegalAccessException("IllegalType: int");
    }
    return ((Integer)theReply).intValue();
  }

  public long longValue() throws IllegalAccessException {
    if (((Byte)theType).byteValue() != Type.TYPE_LONG) {
      throw new IllegalAccessException("IllegalType: long");
    }
    return ((Long)theReply).longValue();
  }

  public float floatValue() throws IllegalAccessException {
    if (((Byte)theType).byteValue() != Type.TYPE_FLOAT) {
      throw new IllegalAccessException("IllegalType: float");
    }
    return ((Float)theReply).floatValue();
  }

  public double doubletValue() throws IllegalAccessException {
    if (((Byte)theType).byteValue() != Type.TYPE_DOUBLE) {
      throw new IllegalAccessException("IllegalType: double");
    }
    return ((Double)theReply).doubleValue();
  }

  public void setType(Class type) {
    theType = type;
  }

  public Object getType() {
    return theType;
  }

  public void setException(Exception e) {
    if (theException == null) {
      theException = e;
    }
  }

  public Exception getException() {
    return theException;
  }
}

