/*
 * Copyright (C) 2003 Ichiro Satoh (ichiro@nii.ac.jp)
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting
 * documentation.
 *
 * FREE USE OF THIS SOFTWARE IS ALLOWED IN ITS "AS IS" CONDITION.
 * BUT WITHOUT WARRANTY. THE AUTHORS DISCLAIM ANY LIABILITY OF ANY 
 * KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS 
 * SOFTWARE.
 */

package tinyrmi.network;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;

import tinyrmi.runtime.loader.MarshalledObject;

public class ChannelClassLoader extends ClassLoader {
	private String defaultPackageStartsWith = "tinyrmi";
	private String defaultClass;
	private String path;
	private ChannelManager manager;
	private Hashtable table;

	public ChannelClassLoader(ChannelManager manager, String defaultClass) {
		this.manager = manager;
		this.defaultClass = defaultClass;
		table = new Hashtable();
	}

	private Class getClass(String classname) throws ClassNotFoundException {
		Class cls = null;
		try {
			if (classname.startsWith(defaultClass)) {
				String filename =
					classname.replace('.', File.separatorChar) + ".class";
				InputStream is = new FileInputStream(filename);
				int length = is.available();
				if (length == 0) {
					throw new IOException(filename);
				}
				byte[] buf = new byte[length];
				is.read(buf);
				is.close();
				cls = defineClass(classname, buf, 0, buf.length);
			}
			else {
				MarshalledObject obj = manager.fetchClass(classname);
				cls = (Class)obj.get();
			}
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		if (cls == null) {
			throw new ClassNotFoundException(classname);
		}
		return cls;
	}

	protected synchronized Class loadClass(String name, boolean resolve)
		throws ClassNotFoundException {
		Class cls = (Class) table.get(name);
		if (name.startsWith(defaultPackageStartsWith) && 
				!name.startsWith(defaultClass)) {
			cls = findSystemClass(name);
		}
		if (cls == null) {
			try {
				cls = getClass(name);
			}
			catch (Exception e) {}
		}
		if (cls == null) {
			cls = findSystemClass(name);
		}
		if (cls == null) {
			throw new ClassNotFoundException(name);
		}
		else {
			table.put(name, cls);
			if (resolve) {
				resolveClass(cls);
			}
			return cls;
		}
	}
}
