/*
 * Copyright (C) 2003 Ichiro Satoh (ichiro@nii.ac.jp)
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 *
 * FREE USE OF THIS SOFTWARE IS ALLOWED IN ITS "AS IS" CONDITION.
 * BUT WITHOUT WARRANTY. THE AUTHORS DISCLAIM ANY LIABILITY OF ANY 
 * KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS 
 * SOFTWARE.
 */

package tinyrmi.network;

import java.net.Socket;
import java.net.InetAddress;
import java.net.UnknownHostException;

public class ChannelId {
  String host = null;
  int port = -1;
  long identify = -1;

  /**
   * @param host the string representing the host address
   * @param port the int representing the port number
   */
  public ChannelId(String host, int port) {
    this.host = host;
    this.port = port;
    try {
      InetAddress iaddr = InetAddress.getByName(host);
      identify = (((long)iaddr.hashCode()) << 16) + port;
    }
    catch (UnknownHostException e) {
    }
  }

  /**
   * @param sock the connecting socket
   */
  public ChannelId(Socket sock) {
    port = sock.getPort();
    InetAddress iaddr = sock.getInetAddress();
    host = iaddr.getHostAddress();
    identify = (((long)iaddr.hashCode()) << 16) + port;
  }

  /**
   * @return the identity
   */
  public long getID() {
    return identify;
  }

  /**
   * @return the host
   */
  public String getHost() {
    return host;
  }

  /**
   * @return the port
   */
  public int getPort() {
    return port;
  }

  /**
   * @return the hashcode
   */
  public int hashCode() {
    return (new Long(identify)).hashCode();
  }

  /**
   * @param element other NodeID to be compared to
   * @return true if bot have the same identity
   */
  public boolean equals(Object obj) {
    if (obj instanceof ChannelId) {
      return toString().equals(obj.toString());
    }
    return false;
  }

  /**
   * @return a printable form of the NodeID
   */
  public String toString() {
    return new String(Long.toString(identify));
  }

  /**
   * @return a printable form of the NodeID in a hex format
   */
  public String toHexString() {
    return new String(Long.toHexString(identify));
  }
}

