/*
 * Copyright (C) 2003 Ichiro Satoh (ichiro@nii.ac.jp)
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting
 * documentation.
 *
 * FREE USE OF THIS SOFTWARE IS ALLOWED IN ITS "AS IS" CONDITION.
 * BUT WITHOUT WARRANTY. THE AUTHORS DISCLAIM ANY LIABILITY OF ANY 
 * KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS 
 * SOFTWARE.
 */
 
package tinyrmi.network;

import java.io.IOException;
import java.util.Hashtable;
import tinyrmi.Message;
import tinyrmi.RemoteException;
import tinyrmi.runtime.RuntimeManager;
import tinyrmi.runtime.loader.MarshalledObject;

public class ChannelManager {
	protected Hashtable registry;
	protected Hashtable targets;
	protected Hashtable bindings;
	protected Channel channel;
	protected	RuntimeManager runtime; 

	public ChannelManager(Channel channel) {
		registry = new Hashtable();
		targets = new Hashtable();
		bindings = new Hashtable();
		runtime = new RuntimeManager(this); // not shared objects
		this.channel = channel;
	}

	public Response handle(Request req) {
		Response res = new Response();
		byte command = req.getCommand();
		//  System.out.println("ChannelManager: code = "+req.getCommand());
		switch (command) {
		case Request.NEWINSTANCE:	{
			try {
				res = runtime.newInstance(req);
			}
			catch (Exception e) {
				res.setException(new RemoteException(e.getMessage()));
				e.printStackTrace();
			}
			break;
		}
		case Request.INVOKE: {
			try {
				res = runtime.invoke(req);
			}
			catch (Exception e) {
				res.setException(new RemoteException(e.getMessage()));
				e.printStackTrace();
			}
			break;
		}
		case Request.FETCH: {
			try {
				res = runtime.fetch(req);
			} 
			catch (Exception e) {
				res.setException(new RemoteException(e.getMessage()));
				e.printStackTrace();
			}
			break;
		}
		default :
			System.out.println("ChannelManager: UNKNOWN");
		}
		return res;
	}
	
	public MarshalledObject fetchClass(String name) throws IOException {
		Request req = new Request();
		req.setCommand(Request.FETCH);
		req.setMessage(new Message(name));
		Response res = channel.call(req);
		System.out.println(
			"ChannelManager: class-fetch = "+ res.getReply().getReply());
		MarshalledObject obj = (MarshalledObject)res.getReply().getReply();
		return obj;
	}
}
