/*
 * Copyright (C) 2003 Ichiro Satoh (ichiro@nii.ac.jp)
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting
 * documentation.
 *
 * FREE USE OF THIS SOFTWARE IS ALLOWED IN ITS "AS IS" CONDITION.
 * BUT WITHOUT WARRANTY. THE AUTHORS DISCLAIM ANY LIABILITY OF ANY 
 * KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS 
 * SOFTWARE.
 */

package tinyrmi.network;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Hashtable;
import tinyrmi.Message;
import tinyrmi.ObjectId;
import tinyrmi.RemoteException;

public class EndPoint implements Runnable {
	final static public int SOCKET_TIMEOUT = 5000;
	protected ServerSocket listenSocket = null;
	protected Hashtable table = null;

	public EndPoint(int port, boolean flag) throws IOException {
		table = new Hashtable();
		if (flag) {
			listenSocket = new ServerSocket(port);
			(new Thread(this)).start();
		}
	}

	public void run() {
		try {
			if (listenSocket != null) {
				Socket clientSock = listenSocket.accept();
				ChannelId nd = new ChannelId(clientSock);
				Channel ch = (Channel) table.get(nd);
				if (ch == null) {
					OutputStream out =
						new BufferedOutputStream(clientSock.getOutputStream());
					InputStream in =
						new BufferedInputStream(clientSock.getInputStream());
					ch = new Channel(in, out);
					table.put(nd, ch);
					(new Thread(ch)).start();
				}
			}
		}
		catch (IOException e) {
			e.printStackTrace();
		}
	}

	private Channel getChannel(String host, int port) throws IOException {
		ChannelId nd = new ChannelId(host, port);
		Channel ch = (Channel) table.get(nd);
		if (ch == null) {
			Socket sock = new Socket(host, port);
			OutputStream out = new BufferedOutputStream(sock.getOutputStream());
			InputStream in = new BufferedInputStream(sock.getInputStream());
			ch = new Channel(in, out);
			table.put(nd, ch);
			(new Thread(ch)).start();
		}
		return ch;
	}

	public ObjectId newInstance(String host, int port, Message msg)
		throws RemoteException, IOException {
		try {
			Channel ch = getChannel(host, port);
			Request req = new Request();
			req.setMessage(msg);
			req.setCommand(Request.NEWINSTANCE);
			System.out.println("EndPoint: creating an instance of "+msg.getName()+
												 " at "+host+":"+port);
			Response res = ch.call(req);
			return (ObjectId)res.getReply().getReply();
		}
		catch (IOException e) {
			throw e;
		}
		catch (Exception e) {
			e.printStackTrace();
			throw new RemoteException(e.toString());
		}
	}

	public Object invoke(String host, int port, ObjectId oid, Message msg)
		throws RemoteException, IOException {
		try {
			Channel ch = getChannel(host, port);
			Request req = new Request();
			req.setID(oid);
			req.setMessage(msg);
			req.setCommand(Request.INVOKE);
			System.out.println("EndPoint: invoking: method ["+msg.getName()+"] of "+oid+
												 " at "+host+":"+port);
			Response res = ch.call(req);
			return (Object) res.getReply().getReply();
		}
		catch (IOException e) {
			throw e;
		}
		catch (Exception e) {
			e.printStackTrace();
			throw new RemoteException(e.toString());
		}
	}

	public static void main(String[] args) throws Exception {
		int port = 80;
		if (args.length == 0) {
			try {
				EndPoint ep = new EndPoint(5000, false);
			}
			catch (IOException e) {
				e.printStackTrace();
			}
		}
		else if (args.length == 1) {
			System.out.println("EndPoint: args.length " + args.length);
			port = Integer.parseInt(args[0]);
			try {
				EndPoint ep = new EndPoint(port, true);
				try {
					Thread.sleep(10000);
				}
				catch (Exception e) {}
				Message msg1 = new Message("java.awt.Frame");
				ObjectId oid = ep.newInstance("localhost", port, msg1);
				Message msg2 = new Message(oid, "show");
				ep.invoke("localhost", port, oid, msg2);
			}
			catch (IOException e) {
				e.printStackTrace();
			}
		}
		else {
			System.out.println("EndPoint: args.length " + args.length);
			port = Integer.parseInt(args[1]);
			try {
				EndPoint ep = new EndPoint(port, false);
				Message msg1 = new Message("java.awt.Frame");
				ObjectId oid = ep.newInstance(args[0], port, msg1);
				System.out.println("EndPoint: oid = " + oid);
				Message msg2 = new Message(oid, "add");
				msg2.setArg(new java.awt.Button("OK"));
				ep.invoke(args[0], port, oid, msg2);
				Message msg3 = new Message(oid, "pack");
				ep.invoke(args[0], port, oid, msg3);
				Message msg4 = new Message(oid, "show");
				ep.invoke(args[0], port, oid, msg4);
			}
			catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
}
