/*
 * Copyright (C) 2003 Ichiro Satoh (ichiro@nii.ac.jp)
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting
 * documentation.
 *
 * FREE USE OF THIS SOFTWARE IS ALLOWED IN ITS "AS IS" CONDITION.
 * BUT WITHOUT WARRANTY. THE AUTHORS DISCLAIM ANY LIABILITY OF ANY 
 * KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS 
 * SOFTWARE.
 */
 
package tinyrmi.network;

import java.io.Serializable;
import java.io.IOException;
import tinyrmi.runtime.loader.MarshalledObject;
import tinyrmi.Message;
import tinyrmi.ObjectId;

public class Request implements Serializable {
  static final byte NEWINSTANCE = 0x01;
  static final byte INVOKE = 0x02;
  static final byte LOOKUP = 0x03;
  static final byte FETCH = 0x04;
  static final byte ADD = 0x05;
  static final byte REMOVE = 0x06;
  static final byte CALLBACK = 0x07;
  static final byte DATA = 0x08;

  protected byte command = -1;
  protected boolean finished = false;
  protected Message msg = null;
  protected ObjectId oid;
  transient protected Response res = null;
  transient MarshalledObject mobj = null;

  public Request() {}

  public void setID(ObjectId oid) {
    this.oid = oid;
  }

  public ObjectId getID() {
    return oid;
  }

  public void setCommand(byte command) {
    this.command = command;
  }

  public byte getCommand() {
    return command;
  }

  public void setResponse(Response res) {
    this.res = res;
  }

  public Response getResponse() {
    return res;
  }

  public void setMessage(Message msg) {
    this.msg = msg;
  }

  public Message getMessage() {
    if (mobj != null) {
      try {
	return (Message)mobj.get();
      }
      catch (IOException e) {
	e.printStackTrace();
      }
      catch (ClassNotFoundException e) {
	e.printStackTrace();
      }
    }
    return msg;
  }

  public Message getMessage(ClassLoader loader) {
    try {
      return (Message)mobj.get(loader);
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    catch (ClassNotFoundException e) {
      e.printStackTrace();
    }
    return null;
  }

  public void setFinished(boolean finished) {
    this.finished = finished;
  }

  public boolean getFinished() {
    return finished;
  }

  public String toString() {
    String line = "Request[";
    switch(command) {
    case NEWINSTANCE:
      line = line + "NEWINSTANCE";
      break;
    case INVOKE:
      line = line + "INVOKE";
      break;
    case LOOKUP:
      line = line + "LOOKUP";
      break;
    case FETCH:
      line = line + "FETCH";
      break;
    default:
    }
    line = line + "]";
    line = line + msg;
    return line;
  }

  private synchronized void readObject(java.io.ObjectInputStream in)
  throws IOException, ClassNotFoundException {
    command = in.readByte();
    finished = in.readBoolean();
    oid = (ObjectId)in.readObject();
    mobj = (MarshalledObject)in.readObject();
  }

  private synchronized void writeObject(java.io.ObjectOutputStream out)
  throws IOException {
    out.writeByte(command);
    out.writeBoolean(finished);
    out.writeObject(oid);
    MarshalledObject mobj = new MarshalledObject(msg);
    out.writeObject(mobj);
  }
}


