/*
 * Copyright (C) 2003 Ichiro Satoh (ichiro@nii.ac.jp)
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting
 * documentation.
 *
 * FREE USE OF THIS SOFTWARE IS ALLOWED IN ITS "AS IS" CONDITION.
 * BUT WITHOUT WARRANTY. THE AUTHORS DISCLAIM ANY LIABILITY OF ANY 
 * KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS 
 * SOFTWARE.
 */
 
package tinyrmi.network;

import java.io.IOException;
import java.io.Serializable;

import tinyrmi.Reply;

public class Response implements Serializable {
  static final int DONE = 0x41;
  static final int UNKNOWN = 0x42;
  static final int ERROR = 0x43;

  protected byte command = -1;
//protected boolean finished = false;
  protected Reply rpy = null;
  protected Exception ex = null;

  public Response() {}

  public Response(Reply rpy) {
    this.rpy = rpy;
  }

  public void setCommand(byte command) {
    this.command = command;
  }

  public byte getCommand() {
    return command;
  }

/*
  public void setFinished(boolean finished) {
    this.finished = finished;
  }

  public boolean getFinished() {
    return finished;
  }
*/
  public void setReply(Reply rpy) {
    this.rpy = rpy;
  }

  public Reply getReply() {
    return rpy;
  }

  public void setException(Exception ex) {
    this.ex = ex;
  }

  public Exception getException() {
    return ex;
  }

  public String toString() {
    String line = "Response[";
    if (rpy != null) {
      line = line + rpy;
    }
    if (ex != null) {
      line = line + ex;
    }
    line = line + "]";
    return line;
  }

  private synchronized void readObject(java.io.ObjectInputStream in)
  throws IOException, ClassNotFoundException {
    command = in.readByte();
    rpy = (Reply)in.readObject();
    ex = (Exception)in.readObject();
  }

  private synchronized void writeObject(java.io.ObjectOutputStream out)
  throws IOException {
    out.writeByte(command);
    out.writeObject(rpy);
    out.writeObject(ex);
  }
}


