/*
 * Copyright (C) 2003 Ichiro Satoh (ichiro@nii.ac.jp)
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting
 * documentation.
 *
 * FREE USE OF THIS SOFTWARE IS ALLOWED IN ITS "AS IS" CONDITION.
 * BUT WITHOUT WARRANTY. THE AUTHORS DISCLAIM ANY LIABILITY OF ANY 
 * KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS 
 * SOFTWARE.
 */
 
package tinyrmi.runtime;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Hashtable;

import tinyrmi.Message;
import tinyrmi.NotBoundException;
import tinyrmi.ObjectId;
import tinyrmi.Reply;
import tinyrmi.network.ChannelClassLoader;
import tinyrmi.network.ChannelManager;
import tinyrmi.network.Request;
import tinyrmi.network.Response;
import tinyrmi.runtime.loader.MarshalledObject;

public class RuntimeManager {
	Hashtable bindings = new Hashtable();
	Hashtable registry = new Hashtable();
	Object obj = null;
	Object handler = null;
	ChannelManager manager = null;
  static private final String DefaultHandler = "tinyrmi.runtime.RuntimeImpl";

	public RuntimeManager(ChannelManager manager) {
		this.manager = manager;
	}

	public Response newInstance(Request req) {
		Response res = new Response();
		try {
			ClassLoader loader = new ChannelClassLoader(manager, DefaultHandler);
			Class cls = loader.loadClass(DefaultHandler);
			Object rt = cls.newInstance();
			Message msg = req.getMessage(loader);
			Class[] classes = {msg.getClass()};
			Object[] objects = {msg};
			Method m = cls.getMethod("newInstance", classes);
			Object obj = m.invoke(rt, objects);
			ObjectId oid = new ObjectId(obj);
			registry.put(oid, rt);
			System.out.println("RuntimeManager: created an instance of "+msg.getName()+
												 " oid = "+oid);
			res.setReply(new Reply(oid));
		}
		catch (InstantiationException e) {
			res.setException(e);
			e.printStackTrace();
		}
		catch (ClassNotFoundException e) {
			res.setException(e);
			e.printStackTrace();
		}
		catch (Exception e) {
			res.setException(e);
			e.printStackTrace();
		}
		return res;
	}

	public Response invoke(Request req) {
		Response res = new Response();
		try {
			Object rt = registry.get(req.getID());
			if (rt == null) {
				throw new NotBoundException();
			}
			ClassLoader loader = rt.getClass().getClassLoader();
			Message msg = req.getMessage(loader);
			Class[] classes = {msg.getClass()};
			Object[] objects = {msg};
			Method meth = rt.getClass().getMethod("invoke", classes);
			Object obj = meth.invoke(rt, objects);
			System.out.println("RuntimeManager: invoked method ["+msg.getName()+
												 "] => "+obj);
			res.setReply(new Reply(obj));
		}
		catch (InvocationTargetException e) {
			e.printStackTrace();
		}
		catch (Exception e) {
			res.setException(e);
			e.printStackTrace();
		}
		return res;
	}

	public Response fetch(Request req) {
		Response res = new Response();
		try {
			Message msg = req.getMessage();
			Class cl = Class.forName(msg.getName());
			System.out.println("RuntimeManager: fetched "+cl);
			MarshalledObject obj = new MarshalledObject(cl);
			res.setReply(new Reply(obj));
		} 
		catch (Exception e) {
			res.setException(e);
			e.printStackTrace();
		}
		return res;
	}
}
