/*
 * Copyright (C) 2003 Ichiro Satoh (ichiro@nii.ac.jp)
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting
 * documentation.
 *
 * FREE USE OF THIS SOFTWARE IS ALLOWED IN ITS "AS IS" CONDITION.
 * BUT WITHOUT WARRANTY. THE AUTHORS DISCLAIM ANY LIABILITY OF ANY 
 * KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS 
 * SOFTWARE.
 */

package tinyrmi.runtime.loader;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InvalidClassException;
import java.io.NotSerializableException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OptionalDataException;
import java.io.StreamCorruptedException;

public class ByteUtil {

  public static Object byteArrayToObject(byte[] data) 
  throws ClassNotFoundException, InvalidClassException, 
  StreamCorruptedException, OptionalDataException, IOException {
    return byteArrayToObject(null, data);
  }

  public static Object byteArrayToObject(ClassLoader loader, byte[] data) 
  throws ClassNotFoundException, InvalidClassException, 
  StreamCorruptedException, OptionalDataException, IOException {
    Object newObject = null;
    ByteArrayInputStream bytein = new ByteArrayInputStream(data);
    ObjectInputStream objin = null;
    if (loader != null) {
      objin = new MarshalInputStream(bytein, loader);
    }
    else {
      objin = new ObjectInputStream(bytein);
    }
    newObject = objin.readObject();
    objin.close();
    bytein.close();
    return newObject;
  }

  public static byte[] objectToByteArray(Object obj)
  throws NotSerializableException {
    try {
      byte[] data = null;
      ByteArrayOutputStream byteout = new ByteArrayOutputStream();
      ObjectOutputStream objout = new MarshalOutputStream(byteout);
      objout.writeObject(obj);
      objout.flush();
      data = byteout.toByteArray();
      objout.close();
      byteout.close();
      return data;
    }
    catch (IOException e) {
      throw new NotSerializableException(""+e);
    }
  }
}


