/*
 * Copyright (C) 2003 Ichiro Satoh (ichiro@nii.ac.jp)
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting
 * documentation.
 *
 * FREE USE OF THIS SOFTWARE IS ALLOWED IN ITS "AS IS" CONDITION.
 * BUT WITHOUT WARRANTY. THE AUTHORS DISCLAIM ANY LIABILITY OF ANY 
 * KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS 
 * SOFTWARE.
 */
 
package tinyrmi.runtime.loader;

import java.io.IOException;
import java.io.Serializable;
import java.io.NotSerializableException;
import java.io.OptionalDataException;
import java.io.StreamCorruptedException;
import java.io.InvalidClassException;

public class MarshalledObject implements Serializable {
  private int hash;
  private byte[] data;

  public MarshalledObject(Object obj) {
    try {
      hash = obj.hashCode();
      data = ByteUtil.objectToByteArray(obj);
    }
    catch (NotSerializableException e) {
      e.printStackTrace();
    }
  }

  public Object get(ClassLoader loader) 
  throws java.io.IOException, java.lang.ClassNotFoundException {
    try {
      return ByteUtil.byteArrayToObject(loader, data);
    }
    catch (InvalidClassException e) {
      e.printStackTrace();
    }
    catch (StreamCorruptedException e) {
      e.printStackTrace();
    }
    catch (OptionalDataException e) {
      e.printStackTrace();
    }
    return null;
  }

  public Object get() 
  throws java.io.IOException, java.lang.ClassNotFoundException {
    try {
      ClassLoader loader = this.getClass().getClassLoader();
      return ByteUtil.byteArrayToObject(loader, data);
    }
    catch (InvalidClassException e) {
      e.printStackTrace();
    }
    catch (StreamCorruptedException e) {
      e.printStackTrace();
    }
    catch (OptionalDataException e) {
      e.printStackTrace();
    }
    return null;
  }

  public int hashCode() {
    return hash + data.length;
  }
  
  public byte[] toByteArray() {
    return data;
  }

  public String toString() {
		String msg = null;
		try {
			msg = "["+get()+"]";
		}
    catch (Exception e) {
      e.printStackTrace();
    }
		return msg;
	}

  public boolean equals(Object obj) {
    if (obj == this) {
      return true;
    }
    if (obj != null && obj instanceof MarshalledObject) {
      MarshalledObject other = (MarshalledObject)obj;
      byte[] otherBytes = other.toByteArray();
      if (otherBytes != null) {
				if (data.length == other.toByteArray().length) {
					for (int i = 0 ; i < otherBytes.length ; i++ ) {
						if (data[i] != otherBytes[i]) {
							return false;
						}
					}
					return true;
				}
      }
    }
    return false;
  }

  private synchronized void readObject(java.io.ObjectInputStream in)
  throws IOException, ClassNotFoundException {
    hash = in.readInt();
/*
    int len = in.readShort();
    System.out.println("MarshalInput: len = "+len);
    data = new byte[len];
    int size = in.read(data, 0, len);
    System.out.println("MarshalInput: size= "+size);
*/
    data = (byte[])in.readObject();
//    System.out.println("MarshalInput: len = "+data.length);
  }

  private synchronized void writeObject(java.io.ObjectOutputStream out)
  throws IOException {
    out.writeInt(hash);
/*
    out.writeShort(data.length);
    out.write(data, 0, data.length);
*/
    out.writeObject(data);
//    System.out.println("MarshalOutput: len = "+data.length);
  }
}
