/*
 * Copyright (C) 2003 Ichiro Satoh (ichiro@nii.ac.jp)
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting
 * documentation.
 *
 * FREE USE OF THIS SOFTWARE IS ALLOWED IN ITS "AS IS" CONDITION.
 * BUT WITHOUT WARRANTY. THE AUTHORS DISCLAIM ANY LIABILITY OF ANY 
 * KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS 
 * SOFTWARE.
 */
 
package tinyrmi.runtime.loader;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InvalidClassException;
import java.io.ObjectInputStream;
import java.io.OptionalDataException;
import java.io.StreamCorruptedException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

public class ObjectHandler {
  private byte[] data;
  
  public Object create(String aClassName) {
    Object anObject = null;
    try {
      Class aClass = Class.forName(aClassName);
      anObject = aClass.newInstance();
			System.out.println("ObjectHandler: anObject = "+anObject);
    }
    catch (ClassNotFoundException e) {
      e.printStackTrace();
    }
    catch (InstantiationException e) {
      e.printStackTrace();
    }
    catch (IllegalAccessException e) {
      e.printStackTrace();
    }
    return anObject;
  }

  public Object create(String aClassName, Object[] objects) {
    Object anObject = null;
    try {
      Class aClass = Class.forName(aClassName);
      anObject = aClass.newInstance();
			System.out.println("ObjectHandler: anObject = "+anObject);
    }
    catch (ClassNotFoundException e) {
      e.printStackTrace();
    }
    catch (InstantiationException e) {
      e.printStackTrace();
    }
    catch (IllegalAccessException e) {
      e.printStackTrace();
    }
    return anObject;
  }

  public ObjectHandler() {}

  public ObjectHandler(byte[] data) {
    this.data = data;
  }

  public Object resume(byte[] data) {
    Object anObject = null;
    try {
      ByteArrayInputStream bytein = new ByteArrayInputStream(data);
      ObjectInputStream objin = new ObjectInputStream(bytein);
      anObject = objin.readObject();
      objin.close();
      bytein.close();
    }
    catch (ClassNotFoundException e) {
      e.printStackTrace();
    }
    catch (InvalidClassException e) {
      e.printStackTrace();
    }
    catch (StreamCorruptedException e) {
      e.printStackTrace();
    }
    catch (OptionalDataException e) {
      e.printStackTrace();
    }
    catch (IOException e) {
      e.printStackTrace();
    }
    return anObject;
  }

  public Object invoke(Object targetObject, String methodName, 
											 Object[] objects) {
    Object anObject = null;
    try {
      if (objects != null) {
				Class[] classes = new Class[objects.length];
				for (int i = 0 ; i < objects.length ; i++ ) {
					classes[i] = objects[i].getClass();
				}
				System.out.println("ClassLoader: invoke("+targetObject.getClass()+
													 ", "+methodName+")");
				Method meth = getMethod(targetObject.getClass(), 
																methodName, classes, false);
				anObject = meth.invoke(targetObject, objects);
      }
      else {
				Class[] classes = {};
				Method meth = getMethod(targetObject.getClass(), 
																methodName, classes, false);
				anObject = meth.invoke(targetObject, objects);
      }
    }
    catch (InvocationTargetException e) {
      e.printStackTrace();
    }
    catch (NoSuchMethodException e) {
      e.printStackTrace();
    }
    catch (IllegalAccessException e) {
      e.printStackTrace();
    }
    return anObject;
  }
	
  protected Method getMethod(Class targetClass, String methodName, 
														 Class[] classes, boolean asStatic) 
		throws NoSuchMethodException {
    int length = 0;
    if (classes != null) {
      length = classes.length;
    }
    Method[] meths = targetClass.getMethods();
    for( int i = 0 ; i < meths.length ; i++ ) {
      if (meths[i].getName().equals("addButton")) {
				System.out.println("ObjectLoader: method["+i+"] = "+meths[i]);
      }
      if (meths[i].getName().equals(methodName) &&
          meths[i].getParameterTypes().length == length &&
          (!asStatic || (meths[i].getModifiers() & Modifier.STATIC) != 0)) {
        Class[] methodArgClasses = meths[i].getParameterTypes();
        boolean isMatch = true;
        for(int j = 0 ; isMatch && j < length ; j++ ) {
          isMatch = isMatch && 
						(classes[j] == null ||
						 methodArgClasses[j].equals(classes[j]) ||
						 methodArgClasses[j].isAssignableFrom(classes[j]));
        }
        if (isMatch) {
          return meths[i];
				}
      }
    }
    throw new NoSuchMethodException(methodName + " in " + targetClass);
  }
}

