/*
 * Copyright (C) 2003 Ichiro Satoh (ichiro@nii.ac.jp)
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting
 * documentation.
 *
 * FREE USE OF THIS SOFTWARE IS ALLOWED IN ITS "AS IS" CONDITION.
 * BUT WITHOUT WARRANTY. THE AUTHORS DISCLAIM ANY LIABILITY OF ANY 
 * KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS 
 * SOFTWARE.
 */

package tinyrmi.runtime.loader;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

public class ObjectLoader {
	static private final String DefaultLoaderHandler =
		"tinyrmi.runtime.loader.ObjectHandler";

	protected static String getDefaultLoaderHandler() {
		return DefaultLoaderHandler;
	}

	public static Object newInstance(ClassLoader loader, String classname)
		throws InstantiationException, ClassNotFoundException {
		Object anObject = null;
		try {
			Object handler =
				loader.loadClass(DefaultLoaderHandler).newInstance();
			Class[] argClasses = { classname.getClass()};
			Object[] argObjects = { classname };
			Method mth = handler.getClass().getMethod("create", argClasses);
			anObject = mth.invoke(handler, (Object[]) argObjects);
		}
		catch (InvocationTargetException e) {
			e.printStackTrace();
		}
		catch (NoSuchMethodException e) {
			e.printStackTrace();
		}
		catch (IllegalAccessException e) {
			e.printStackTrace();
		}
		catch (IllegalArgumentException e) {
			e.printStackTrace();
		}
		return anObject;
	}

	public static Object newInstance(
		ClassLoader loader,
		String classname,
		Object[] objects)
		throws InstantiationException, ClassNotFoundException {
		Object anObject = null;
		try {
			Object handler =
				loader.loadClass(DefaultLoaderHandler).newInstance();
			Object[] objs = new Object[objects.length];
			Class[] argClasses = { classname.getClass(), objs.getClass()};
			Object[] argObjects = { classname, objects };
			Method mth = handler.getClass().getMethod("create", argClasses);
			anObject = mth.invoke(handler, (Object[]) argObjects);
		}
		catch (InvocationTargetException e) {
			e.printStackTrace();
		}
		catch (NoSuchMethodException e) {
			e.printStackTrace();
		}
		catch (IllegalAccessException e) {
			e.printStackTrace();
		}
		catch (IllegalArgumentException e) {
			e.printStackTrace();
		}
		return anObject;
	}

	public static Object invoke(
		ClassLoader loader,
		Object targetObject,
		String methodName,
		Class[] classes,
		Object[] objects)
		throws InvocationException {
		Object result = null;
		try {
			Class[] argClasses =
				{
					targetObject.getClass(),
					methodName.getClass(),
					objects.getClass()};
			Object[] argObjects = { targetObject, methodName, objects };
			Class c = loader.loadClass(DefaultLoaderHandler);
			System.out.println("ObjectLoader c = " + c);
			Method meth =
				getMethod(
					loader.loadClass(DefaultLoaderHandler),
					"invoke",
					argClasses,
					false);
			Object handler =
				loader.loadClass(DefaultLoaderHandler).newInstance();
			result = meth.invoke(handler, (Object[]) argObjects);
		}
		catch (NoSuchMethodException e) {
			throw new InvocationException(e.getMessage());
		}
		catch (IllegalAccessException e) {
			throw new InvocationException(e.getMessage());
		}
		catch (IllegalArgumentException e) {
			throw new InvocationException(e.getMessage());
		}
		catch (NullPointerException e) {
			throw new InvocationException(e.getMessage());
		}
		catch (InvocationTargetException e) {
			throw new InvocationException(e.getMessage());
		}
		catch (Exception e) {
			throw new InvocationException(e.getMessage());
		}
		return result;
	}

	static public Object invoke(
		Object targetObject,
		String methodName,
		Class[] classes,
		Object[] objects)
		throws InvocationException {
		Object result = null;
		try {
			Method meth =
				getMethod(targetObject.getClass(), methodName, classes, false);
			result = meth.invoke(targetObject, objects);
		}
		catch (NoSuchMethodException e) {
			throw new InvocationException(e.getMessage());
		}
		catch (IllegalAccessException e) {
			throw new InvocationException(e.getMessage());
		}
		catch (IllegalArgumentException e) {
			throw new InvocationException(e.getMessage());
		}
		catch (NullPointerException e) {
			throw new InvocationException(e.getMessage());
		}
		catch (InvocationTargetException e) {
			throw new InvocationException(e.getMessage());
		}
		catch (Exception e) {
			throw new InvocationException(e.getMessage());
		}
		return result;
	}

	protected static Method getMethod(
		Class targetClass,
		String methodName,
		Class[] classes,
		boolean asStatic)
		throws NoSuchMethodException {
		int length = 0;
		if (classes != null) {
			length = classes.length;
		}
		Method[] meths = targetClass.getMethods();
		for (int i = 0; i < meths.length; i++) {
			if (meths[i].getName().equals(methodName)
				&& meths[i].getParameterTypes().length == length
				&& (!asStatic
					|| (meths[i].getModifiers() & Modifier.STATIC) != 0)) {
				Class[] methodArgClasses = meths[i].getParameterTypes();
				boolean isMatch = true;
				for (int j = 0; isMatch && j < length; j++) {
					isMatch =
						isMatch
							&& (classes[j] == null
								|| methodArgClasses[j].equals(classes[j])
								|| methodArgClasses[j].isAssignableFrom(
									classes[j]));
				}
				if (isMatch) {
					return meths[i];
				}
			}
		}
		throw new NoSuchMethodException(methodName + " in " + targetClass);
	}
}
