/*
 * Copyright (C) 2003 Ichiro Satoh (ichiro@nii.ac.jp)
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting
 * documentation.
 *
 * FREE USE OF THIS SOFTWARE IS ALLOWED IN ITS "AS IS" CONDITION.
 * BUT WITHOUT WARRANTY. THE AUTHORS DISCLAIM ANY LIABILITY OF ANY 
 * KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS 
 * SOFTWARE.
 */

package tinyrmi.runtime.loader;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.InvocationTargetException;
import java.io.InvalidClassException;
import java.io.OptionalDataException;
import java.io.StreamCorruptedException;
import java.io.IOException;
import tinyrmi.Message;

public class ObjectRuntime {
	Object obj;
	ClassLoader loader = null;

	public ObjectRuntime() {
		loader = this.getClass().getClassLoader();
	}

	public ObjectRuntime(ClassLoader loader) {
		this.loader = loader;
	}

	public Object newInstance(byte[] data)
		throws
			NoSuchMethodException,
			InstantiationException,
			ClassNotFoundException,
			InvocationTargetException,
			IllegalAccessException,
			IllegalArgumentException {
		Message msg = null;
		try {
			msg = (Message) ByteUtil.byteArrayToObject(loader, data);
		}
		catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		catch (InvalidClassException e) {
			e.printStackTrace();
		}
		catch (StreamCorruptedException e) {
			e.printStackTrace();
		}
		catch (OptionalDataException e) {
			e.printStackTrace();
		}
		catch (IOException e) {
			e.printStackTrace();
		}
		Class cl = Class.forName(msg.getName());
		Constructor cst = cl.getConstructor(msg.getTypes());
		obj = cst.newInstance(msg.getArgs());
		return obj;
	}

	public Object invoke(byte[] data)
		throws
			InvocationTargetException,
			NoSuchMethodException,
			IllegalAccessException,
			IllegalArgumentException {
		Object anObject = null;
		Message msg = null;
		try {
			msg = (Message) ByteUtil.byteArrayToObject(loader, data);
		}
		catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		catch (InvalidClassException e) {
			e.printStackTrace();
		}
		catch (StreamCorruptedException e) {
			e.printStackTrace();
		}
		catch (OptionalDataException e) {
			e.printStackTrace();
		}
		catch (IOException e) {
			e.printStackTrace();
		}
		Class cl = obj.getClass();
		Method meth = getMethod(cl, msg.getName(), msg.getTypes(), false);
		return meth.invoke(obj, msg.getArgs());
	}

	protected static Method getMethod(
		Class targetClass,
		String methodName,
		Class[] classes,
		boolean asStatic)
		throws NoSuchMethodException {
		int length = 0;
		if (classes != null) {
			length = classes.length;
		}
		Method[] meths = targetClass.getMethods();
		for (int i = 0; i < meths.length; i++) {
			if (meths[i].getName().equals(methodName)
				&& meths[i].getParameterTypes().length == length
				&& (!asStatic
					|| (meths[i].getModifiers() & Modifier.STATIC) != 0)) {
				Class[] methodArgClasses = meths[i].getParameterTypes();
				boolean isMatch = true;
				for (int j = 0; isMatch && j < length; j++) {
					isMatch =
						isMatch
							&& (classes[j] == null
								|| methodArgClasses[j].equals(classes[j])
								|| methodArgClasses[j].isAssignableFrom(
									classes[j]));
				}
				if (isMatch) {
					return meths[i];
				}
			}
		}
		throw new NoSuchMethodException(methodName + " in " + targetClass);
	}
}
